import functools
from abc import ABC
from inspect import signature, Parameter


class FilterABC(ABC):

    def __init__(self):
        ABC.__init__(self)

        self._page_index = 0
        self._page_size = 10
        self._sort_direction = ''
        self._sort_column = ''
        
    @property
    def page_index(self) -> int:
        return self._page_index
    
    @property
    def page_size(self) -> int:
        return self._page_size
    
    @property
    def sort_direction(self) -> str:
        return self._sort_direction
    
    @property
    def sort_column(self) -> str:
        return self._sort_column

    @staticmethod
    def get_filter(f, values: dict):
        sig = signature(f)
        for param in sig.parameters.items():
            parameter = param[1]
            if parameter.name == 'self' or parameter.annotation == Parameter.empty:
                continue

            if issubclass(parameter.annotation, FilterABC):
                filter = parameter.annotation()
                filter.from_dict(values)
                return filter

    @classmethod
    def resolve_filter_annotation(cls, f=None):
        if f is None:
            return functools.partial(cls.filter)

        @functools.wraps(f)
        def decorator(*args, **kwargs):
            if 'filter' in kwargs:
                kwargs['filter'] = cls.get_filter(f, kwargs['filter'])

            return f(*args, **kwargs)

        return decorator
