from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.container import Guild
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.server import Server
from bot_graphql.abc.filter_abc import FilterABC


class ServerFilter(FilterABC):

    def __init__(self):
        FilterABC.__init__(self)

        self._id = None
        self._discord_id = None
        self._name = None

    def from_dict(self, values: dict):
        if 'id' in values:
            self._id = values['id']

    @ServiceProviderABC.inject
    def filter(self, query: List[Server], bot: DiscordBotServiceABC) -> List[Server]:
        result = List(Server)

        if self._id is not None:
            query = query.where(lambda x: x.server_id == self._id)

        if self._discord_id is not None:
            query = query.where(lambda x: x.discord_server_id == self._discord_id)

        if self._name is not None:
            def where_guild(x: Guild):
                guild = bot.get_guild(x.discord_server_id)
                return guild is not None and (self._name.lower() == guild.name.lower() or self._name.lower() in guild.name.lower())

            query = query.where(where_guild)

        skip = self.page_size * self.page_index
        result = query.skip(skip).take(self.page_size)

        return result
